@@ -1,12 +1,12 @@
 ;;; blist.el --- Display bookmarks in an ibuffer way  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2021, 2022, 2023  Free Software Foundation, Inc.
+;; Copyright (C) 2021, 2022, 2023, 2024  Free Software Foundation, Inc.
 
 ;; Author: Durand <durand@jsdurand.xyz>
 ;; Keywords: convenience
 ;; URL: https://gitlab.com/mmemmew/blist
-;; Package-Requires: ((ilist "0.1") (emacs "24"))
-;; Version: 0.3
+;; Package-Requires: ((ilist "0.4") (emacs "24"))
+;; Version: 0.4
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -149,7 +149,7 @@ Used by `ilist-dag' to define an automatic filter group."
        (handler-name
         (cond
          ;; Some special cases
-         ((string-match-p "pdf" handler-name) "PDF") ; to handle pdf-view.
+         ((string-match-p "pdf" handler-name) "PDF")
          ((string-match-p "^el$" handler-name) "ELisp")
          ((<= (length handler-name) 3) (upcase handler-name))
          ((capitalize handler-name))))))))
@@ -405,7 +405,9 @@ list; they are simply ignored."
            (const :tag "Vertically" vertical)
            (const :tag "Horizontally" horizontal)
            (const :tag "Spirally" spiral)
-           (const :tag "A main window along with side splits" main-side)
+           (const
+            :tag "A main window along with side splits"
+            main-side)
            (const :tag "Towards Left" left)
            (const :tag "Towards Right" right)
            (const :tag "Towards Up" up)
@@ -414,7 +416,8 @@ list; they are simply ignored."
 
 ;;;; Edit bookmark annotation buffer name
 
-(defcustom blist-edit-annotation-buffer-name "*Edit Bookmark Annotation*"
+(defcustom blist-edit-annotation-buffer-name
+  "*Edit Bookmark Annotation*"
   "The name of the buffer used for editing bookmark annotation."
   :type 'string)
 
@@ -424,6 +427,11 @@ list; they are simply ignored."
   "If non-nil, show a header of column names as well."
   :type 'boolean)
 
+;;;; Whether to show the number of items
+
+(defcustom blist-show-item-num-p nil
+  "If non-nil, the number of items is shown in the group header.")
+
 ;;; Variables
 
 ;;;; Sorter
@@ -602,7 +610,8 @@ used as a `revert-buffer-function'."
           #'blist-filter-groups
           blist-discard-empty-p
           blist-sorter
-          t))
+          t
+          blist-show-item-num-p))
         (goto-char (point-min))
         ;; set the header if necessary
         (cond
@@ -706,6 +715,7 @@ used as a `revert-buffer-function'."
   (define-key map (vector ?% ?l) #'blist-mark-by-location)
   (define-key map (vector ?j) #'blist-jump-to-line)
   (define-key map (vector ?J) #'blist-jump-to-group)
+  (define-key map (vector ?i) #'blist-show-info)
   (define-key map (vector ?\M-j) #'blist-jump-to-group)
   (define-key map (vector ?\M-g) #'blist-jump-to-line)
   (define-key map (vector ?\M-G) #'blist-jump-to-group))
@@ -722,7 +732,11 @@ used as a `revert-buffer-function'."
 
 ;;;; The range of operation
 
-(defun blist-operate-range (arg &optional use-default-p default-start default-end)
+(defun blist-operate-range (arg
+                            &optional
+                            use-default-p
+                            default-start
+                            default-end)
   "Return the range for the operation.
 If region is active, use the region.
 
@@ -1304,7 +1318,7 @@ progress."
          (name-len (length names))
          (new-len (length new-list))
          (table (make-hash-table
-                 :test 'equal :size (+ name-len new_len)))
+                 :test 'equal :size (+ name-len new-len)))
          (count 0))
     ;; use a hash table so that testing for membership is a constant
     ;; time operation
@@ -1419,7 +1433,16 @@ get unique numeric suffixes \"<2>\", \"<3>\", etc."
   (interactive)
   (let* ((group-header (ilist-get-group))
          (group-symbol (intern (format "[ %s ]" group-header)))
+         (group-items-num
+          (ilist-get-property (point) 'ilist-item-num))
          (hidden-p (ilist-get-property (point) 'blist-hidden))
+         (hidden-ellipsis (cond (hidden-p "") (" ...")))
+         (group-title
+          (cond
+           (blist-show-item-num-p
+            (format "[ %s - %d%s ]\n"
+                    group-header group-items-num hidden-ellipsis))
+           ((format "[ %s%s ]\n" group-header hidden-ellipsis))))
          (inhibit-read-only t))
     (blist-assert-mode)
     (cond
@@ -1436,8 +1459,9 @@ get unique numeric suffixes \"<2>\", \"<3>\", etc."
       (save-excursion
         (insert
          (propertize
-          (format "%s\n" group-symbol)
-          'ilist-group-header group-header)))
+          group-title
+          'ilist-group-header group-header
+          'ilist-item-num group-items-num)))
       (remove-from-invisibility-spec group-symbol))
      ;; not hidden
      ((goto-char (ilist-point-at-eol))
@@ -1447,9 +1471,10 @@ get unique numeric suffixes \"<2>\", \"<3>\", etc."
           (delete-region start end)
           (insert
            (propertize
-            (format "[ %s ... ]\n" group-header)
+            group-title
             'ilist-group-header group-header
-            'blist-hidden t))
+            'blist-hidden t
+            'ilist-item-num group-items-num))
           ;; Emacs has a bug that if an invisible character right next
           ;; to the visible part has a display property, then it will
           ;; turn out to be visible.  So we insert an invisible
@@ -1972,7 +1997,87 @@ stop at."
 
 ;;;; blist-show-info
 
+;; TODO: Some information should be inserted first, with outstanding
+;; faces, and the rest should be inserted at the end.
+
+(defun blist-show-info (&optional arg)
+  "Pop a buffer showing detailed information about the bookmarks.
+If there are marked bookmarks, show the information of those
+bookmarks.
+
+Otherwise, if the point is at a bookmark, show the information of
+that bookmark.  If, and only if, ARG is an integer, show ARG many
+following bookmarks; if ARG is negative, show minus ARG many
+preceding bookmarks instead.
+
+Otherwise, if the point is at a group header, show the
+information of the bookmarks of that group."
+  (interactive "p")
+  (blist-assert-mode)
+  (let* ((marked-items (ilist-map-lines #'ilist-get-index
+                                        #'ilist-is-marked))
+         (items
+          (cond
+           (marked-items)
+           ((ilist-get-group)
+            (let ((start (point))
+                  (end (save-excursion
+                         (ilist-forward-group-header 1)
+                         (point))))
+              (ilist-map-lines
+               #'ilist-get-index #'ilist-get-index start end)))
+           ;; HACK: if not on a normal line, it will return nil, so
+           ;; that this clause is skipped
+           ((delq nil (list (ilist-get-index))))
+           ((user-error "No bookmarks to open"))))
+         (items (mapcar
+                 (lambda (index) (nth index bookmark-alist))
+                 items))
+         (items-len (length items))
+         (seperator (make-string (1- (window-body-width)) ?-))
+         (buffer (get-buffer-create "*blist-info*"))
+         (durand-window-max-height 0.5))
+    (with-current-buffer buffer
+      (erase-buffer)
+      (let ((count 0) temp)
+        (while (consp items)
+          (setq count (1+ count))
+          (setq temp (car items))
+          (setq items (cdr items))
+          (insert "name: " (car temp))
+          (newline)
+          (mapc
+           (lambda (cell)
+             (cond
+              ((and
+                (consp cell)
+                (cdr cell)
+                (eq (car cell) 'last-modified))
+               (insert
+                (format
+                 "%S: [%s]"
+                 (car cell)
+                 (format-time-string "%F %a %R" (cdr cell))))
+               (newline))
+              ((and (consp cell) (cdr cell))
+               (insert (format "%S: %S" (car cell) (cdr cell)))
+               (newline))))
+           temp)
+          (cond ((< count items-len)
+                 (insert seperator)
+                 (newline))))
+        (goto-char (point-min))))
+    (display-buffer
+     buffer
+     (list (list #'display-buffer-in-side-window)
+           (cons 'side 'bottom)
+           (cons 'window-height
+                 #'durand-fit-window-to-buffer-with-max)))))
+
 ;; TODO: Display counts about each groups.
 
+;; TODO: Navigate between each fields.
+
+
 (provide 'blist)
 ;;; blist.el ends here
